@@ -530,4 +530,15 @@ def f():
 If you would like to see debugging output,
 try: %s -d5
 ''' % sys.argv[0])
+
+
+d = {  # comment
+    1: 2
+}
+
+# issue 138
+[
+    12,  # this is a multi-line inline
+         # comment
+]
 #
