@@ -88,4 +88,7 @@ lambda a, b=h[:], c=0: (a, b, c)
 if not -5 < x < +5:
     print >>sys.stderr, "x is out of range."
 print >> sys.stdout, "x is an integer."
+
+if True:
+    *a, b = (1, 2, 3)
 #:
